﻿using System;
using System.Text;
using System.Web.UI;
using System.Web.UI.HtmlControls;

namespace YaneWebSDK.ControlsTiny
{
	public partial class AutoCompleteTextBox : System.Web.UI.UserControl
	{
		protected void Page_Load(object sender, EventArgs e)
		{
			ClientScriptManager csMgr = Page.ClientScript;

			var clientScript = "<script type='text/javascript' src='js/suggest.js'></script>";
			var key = "_suggest"; // 登録名

			if(!csMgr.IsClientScriptBlockRegistered(key))
			{
				csMgr.RegisterClientScriptBlock(this.GetType(), key, clientScript);
			}

			var sb = new StringBuilder();

			// startXXXXというスタートアップ用のJavaScriptを用意する。
			sb.AppendLine("<script type='text/javascript'>");

			sb.AppendLine("window.addEventListener ? window.addEventListener('load', " + ID +
					  "_START , false) : window.attachEvent('onload', " + ID + "_START );");

			// functionの外にIDを出しておくか..
			sb.AppendLine("var " + ID + "_VAR; function " + ID + "_START() { " + ID + "_VAR = new Suggest.Local('" +
			              ID + "_TEXT','" + ID + "_SUGGEST'," +
			              ID + "_LIST,{dispMax: "+ DispItemMax +", interval: "+DispInterval );
			
			if (RelatedTextBox!=null)
			{
				sb.AppendLine(", onSelect : function(index){ " +
						RelatedTextBox + "_VAR.setInputText('');"+
						RelatedTextBox + "_VAR.candidateList = " +
						RelatedTextBox + "_LIST[index];}");
			}

			sb.AppendLine("} ); }");

			sb.AppendLine("var " + ID + "_LIST = [" + ListItems + "];");

			sb.AppendLine("</script>");

			csMgr.RegisterStartupScript(this.GetType(),ID,sb.ToString());

			// CSSの指定。
			var myHtmlLink = new HtmlLink();
			myHtmlLink.Href = cssFileName;
			myHtmlLink.Attributes.Add("rel", "stylesheet");
			myHtmlLink.Attributes.Add("type", "text/css");
			Page.Header.Controls.Add(myHtmlLink);
		}

		/// <summary>
		/// このコントロールに変更があったときに通知したい
		/// コントロールのIDを設定しておくと、そのコントロールの
		/// onSelect(index)にコールバックがかかる。
		/// 
		/// このコントロールで「パソコン」を選んだときに
		/// 別のコントロールでパソコンの機種名を表示したい場合などに使う。
		/// 
		/// </summary>
		public string RelatedTextBox
		{
			get; set;
		}

		/// <summary>
		/// 表示のスキャン間隔。defaultでは100ms
		/// </summary>
		public int DispInterval
		{
			get { return dispInterval;  }
			set { dispInterval = value; }
		}

		private int dispInterval = 100;

		/// <summary>
		/// 最大アイテム表示数。defaultでは20。
		/// </summary>
		public int DispItemMax
		{
			get { return dispItemMax; }
			set { dispItemMax = value; }
		}

		private int dispItemMax = 20;

		/// <summary>
		/// 表示したいアイテムを設定する。
		/// 必ず設定してね。
		/// 
		/// 例 :
//	["有限会社やねうらお","ゆうげんがいしゃやねうらお","yuugengaisyayaneurao"],
//	["有限会社やねうさん","ゆうげんがいしゃやねうさん","yuugengaisyayaneusan"],
//	["株式会社やねうらお","かぶしきがいしゃやねうらお","kabushikigaisyayaneurao"],
//	["株式会社やねうさん","かぶしきがいしゃやねうさん","kabushikigaisyayaneusan"]
		/// 
		/// だとか。
		/// 
		/// RelatedTextBoxで指定されるほうは、
//	[["やねうらお"],["やねうら王"]],
//	[["やねうなんとかさん"],["やねうーさん"],["やねうりゃーさん"]],
//	[["やねうらおっさん"],["やねうら王様"]],
//	[["やねう株なんとかさ"],["やねう株さん"],["やねうりゃ株さん"]]
		/// 
		/// のように括弧がひとつ深くなるので注意。
		/// 
		/// </summary>
		public string ListItems
		{
			get; set;
		}

		/// <summary>
		/// Suggest用のcssファイル名。これで切り替えることが出来る。
		/// </summary>
		public string CssFileName
		{
			get { return cssFileName; }
			set { cssFileName = value;  }

		}
		private string cssFileName = "js/suggest.css";
	}
}